package de.fhkoeln.cvogt.android.propertyanimationandroid;

// Programmierung verteilter und mobiler Anwendungen
// (Programming Distributed and Mobile Applications)
// Cologne University of Applied Sciences
// Institute of Communications Engineering
// 26.2.2015

// This activity demonstrates how to animate an object property through a ValueAnimator and its update listener.

import android.animation.FloatEvaluator;
import android.animation.TimeInterpolator;
import android.animation.TypeEvaluator;
import android.animation.ValueAnimator;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

public class ValueAnimatorAnimationActivity extends ActionBarActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.value_animator_animation);
    }

    public void startAnimation(View v) {

        int duration = 2000;  // duration of the animation in ms
        int screenHeight = getResources().getDisplayMetrics().heightPixels;

        // create a ValueAnimator to control the animation
        ValueAnimator animator = new ValueAnimator();

        // set its TimeInterpolator to map the real time axis to a virtual time axis,
        //  thus "stretching" or "compressing" the time axis (see SelfDefinedTimingAnimation.java for a detailed explanation)
        animator.setInterpolator(new MyTimeInterpolator());

        // set its evaluator to calculate the object property values from the virtual time values
        animator.setEvaluator(new MyFloatEvaluator());
        // alternative: set the evaluator when creating the animator through a static factory method - ValueAnimator.ofObject(typeEvaluator,startValue,targetValue)

        // set the update listener to modify the animated object's property for each animation step
        animator.addUpdateListener(new MyAnimatorUpdateListener());

        // set start and target values of the animation
        TextView animatedView = (TextView) findViewById(R.id.view20);
        animator.setFloatValues(animatedView.getY(),screenHeight-100);
        // alternative: set these values when creating the animator through a static factory method - ValueAnimator.ofFloat(startValue,targetValue)

        // set the duration of the animation
        animator.setDuration(duration);

        // start the animation
        animator.start();

    }

    public void reset(View v) {

        setContentView(R.layout.value_animator_animation);

    }

    // Listener: Its callback function onAnimationUpdate() is called each time a new step of the animation shall be displayed
    //           (normally every 10 ms).

    class MyAnimatorUpdateListener implements ValueAnimator.AnimatorUpdateListener {

        @Override
        public void onAnimationUpdate(ValueAnimator animator) {

            Log.v("MYLOG", "onAnimationUpdate()");

            // calculate new value for the animated property
            float currentPosition = (float) animator.getAnimatedValue();

            // set the animated object's property to the new value
            TextView animatedView = (TextView) findViewById(R.id.view20);
            animatedView.setY(currentPosition);

            // refresh the display
            animatedView.invalidate();

        }

    }

    // The evaluator calculates the current value of the property to be animated,
    //   taking into account the current progress of the animation (given by a fraction parameter between 0.0 and 1.0).
    // (This class is included only for instructional purposes here; it has the same effect as the predefined class FloatEvaluator.)

    class MyFloatEvaluator implements TypeEvaluator<Float> {

        @Override
        public Float evaluate(float fraction, Float startValue, Float endValue) {

            Log.v("MYLOG", "evaluate()");

            // linear interpolation between start and end value
            return (float) (startValue.floatValue() + fraction * (endValue.floatValue() - startValue.floatValue()));

        }

    }

    // The time interpolator maps real time values to virtual time values, which are then fed into the Evaluator function evaluate().
    // (For a detailed comment on time interpolators, see SelfDefinedTimingAnimation.java.)

    class MyTimeInterpolator implements TimeInterpolator {

        @Override
        public float getInterpolation(float input) {

            Log.v("MYLOG", "getInterpolation()");

            // The virtual time is calculated as the square of the real time and will hence be smaller than the real time
            // as long as input<1.0. Therefore, this simple calculation has the effect that the animation starts slower
            // than an animation with constant speed and will then catch up.

            return input*input;

        }

    }

}

